"""Example responses to calling subprocess.run(icacls):

For testing handling of the windows executable 'icacls' without requiring any windows environment to run tests.
"""

from subprocess import CompletedProcess


# making a mistake with parameters. In this case forgetting username
ERROR_INVALID_PARAMETER = CompletedProcess(
    args=["icacls", "C:\\a_directory", "/grant", "(CI)(OI)", "(GA)", "/T"],
    returncode=87,
    stderr=b'Invalid parameter "(CI)(OI)"\r\n',
    stdout=b"",
)

# Process one folder with one file
SUCCESS_2_FILES = CompletedProcess(
    args=[
        "icacls",
        r"C:\temp\icacls",
        "/grant",
        "z428172",
        "(CI)(OI)",
        "(GR,DC)",
        "/T",
    ],
    returncode=0,
    stderr=b"",
    stdout=rb"processed file: C:\\temp\\icacls\r\nprocessed file: C:\\temp\\icacls\\a_file.txt\r\nSuccessfully processed 2 files; Failed processing 0 files\r\n",
)


# Process one folder with one file
ERROR_NON_EXISTENT_FOLDER = CompletedProcess(
    args=[
        "icacls",
        r"C:\\non_existent",
        "/grant",
        "z428172",
        "(CI)(OI)",
        "(GR,DC)",
        "/T",
    ],
    returncode=5,
    stderr=b"C:\\Documents and Settings\\*: Access is denied.\r\n",
    stdout=b"Successfully processed 0 files; Failed processing 1 files\r\n",
)

# This user is not known in the windows environment.
ERROR_UNKNOWN_USER = CompletedProcess(
    args=["icacls", r"C:\\TEMP", "/grant", "unknown_user", "(CI)(OI)", "(GR,DC)", "/T"],
    returncode=1332,
    stderr=b"non_existent_user: No mapping between account names and security IDs was done.\r\n",
    stdout=b"Successfully processed 0 files; Failed processing 1 files\r\n",
)


ERROR_UNKNOWN_RESPONSE_CODE = CompletedProcess(
    args=["icacls", r"C:\\TEMP", "/grant", "unknown_user", "(CI)(OI)", "(GR,DC)", "/T"],
    returncode=12345,
    stderr=b"",
    stdout=b"Something really weird",
)

# I would not put it past this executable to print to standard error but still have SUCCESS return code
SUCCES_BUT_STDERR = CompletedProcess(
    args=["icacls", r"C:\\TEMP", "/grant", "unknown_user", "(CI)(OI)", "(GR,DC)", "/T"],
    returncode=0,
    stderr=b"Everything went WRONG even though returncode is succes",
    stdout=b"",
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from distutils import dir_util
from pathlib import Path
from subprocess import CompletedProcess, CalledProcessError
from unittest.mock import patch

import pytest
from pytest import fixture

from tests import RESOURCE_PATH
from icaclswrap.foldertool import (
    WinFolderPermissionTool,
    ACLToolException,
    InvalidParameterException,
    FolderAccessException,
    UnknownUserException,
)
from icaclswrap.rights import FULL_ACCESS
from tests.icacls_mock_reponses import (
    ERROR_INVALID_PARAMETER,
    ERROR_NON_EXISTENT_FOLDER,
    ERROR_UNKNOWN_USER,
    ERROR_UNKNOWN_RESPONSE_CODE,
    SUCCES_BUT_STDERR,
)


@pytest.fixture
def resources_folder(tmpdir):
    """A one-time copy of a folder containing a dicom file

    Returns
    -------
    str
        path to folder

    """
    template_folder = Path(RESOURCE_PATH) / "test_wrappers_pytest"
    dir_util.copy_tree(str(template_folder), str(tmpdir))
    return tmpdir


@pytest.fixture
def empty_folder(tmpdir_factory):
    """One-time empty folder

    Returns
    -------
    str
        path to folder

    """
    return tmpdir_factory.mktemp(basename="empty")


@pytest.fixture
def acl_tool():
    """A standard instance of the acl tool"""
    return WinFolderPermissionTool()


@pytest.mark.parametrize(
    "run_exception, expected_tool_exception",
    [
        (
            FileNotFoundError("The system cannot find the file specified"),
            ACLToolException,
        ),
        (CalledProcessError(returncode=100, cmd='testcommand'),
         CalledProcessError)
    ],
)
def test_folder_subprocess_exception_handling(
    acl_tool, empty_folder, run_exception, expected_tool_exception
):
    """When subprocess.run raises errors, how are they handled?"""

    with patch("icaclswrap.foldertool.subprocess.run") as mock_run:
        # calling an unknown executable will raise FileNotFoundError
        mock_run.side_effect = run_exception
        with pytest.raises(expected_tool_exception):
            acl_tool.set_rights(
                path=empty_folder, username="testuser", rights_collection=FULL_ACCESS
            )


@pytest.mark.parametrize(
    "run_response, expected_exception_class",
    [
        (ERROR_INVALID_PARAMETER, InvalidParameterException),
        (ERROR_NON_EXISTENT_FOLDER, FolderAccessException),
        (ERROR_UNKNOWN_USER, UnknownUserException),
        (ERROR_UNKNOWN_RESPONSE_CODE, ACLToolException),
        (SUCCES_BUT_STDERR, ACLToolException),
    ],
)
def test_error_responses(run_response, expected_exception_class, acl_tool):
    """When the icacls process responds, but yields some non-0 return code """

    with patch("icaclswrap.foldertool.subprocess.run") as mock_run:
        mock_run.return_value = run_response
        with pytest.raises(expected_exception_class):
            acl_tool.set_rights(path=r"\test", username="testuser", rights_collection=FULL_ACCESS)



# -*- coding: utf-8 -*-
"""Definitions of different rights that a user might have on a folder or file.

"""
from typing import Set


class ScriptParameter:
    """A parameter that can be added to the icacls

    """

    def __init__(self, code, description):
        self.code = code
        self.description = description


class SpecificRight(ScriptParameter):
    pass


class SpecificRights:
    """All rights that can be set

    """

    DELETE = SpecificRight(code="DE", description="delete")
    READ_CONTROL = SpecificRight(code="RC", description="read control")
    WRITE_DAC = SpecificRight(code="WDAC", description="write DAC")
    WRITE_OWNER = SpecificRight(code="WO", description="write owner")
    SYNCHRONIZE = SpecificRight(code="S", description="synchronize")
    ACCESS_SYSTEM_SECURITY = SpecificRight(
        code="AS", description="access system security"
    )
    MAXIMUM_ALLOWED = SpecificRight(code="MA", description="maximum allowed")
    GENERIC_READ = SpecificRight(code="GR", description="generic read")
    GENERIC_WRITE = SpecificRight(code="GW", description="generic write")
    GENERIC_EXECUTE = SpecificRight(code="GE", description="generic execute")
    GENERIC_ALL = SpecificRight(code="GA", description="generic all")
    READ_DATA_LIST_DIRECTORY = SpecificRight(
        code="RD", description="read data/list directory"
    )
    WRITE_DATA_ADD_FILE = SpecificRight(code="WD", description="write data/add file")
    APPEND_DATA_ADD_SUBDIRECTORY = SpecificRight(
        code="AD", description="append data/add subdirectory"
    )
    READ_EXTENDED_ATTRIBUTES = SpecificRight(
        code="REA", description="read extended attributes"
    )
    WRITE_EXTENDED_ATTRIBUTES = SpecificRight(
        code="WEA", description="write extended attributes"
    )
    EXECUTE_TRAVERSE = SpecificRight(code="X", description="execute/traverse")
    DELETE_CHILD = SpecificRight(code="DC", description="delete child")
    READ_ATTRIBUTES = SpecificRight(code="RA", description="read attributes")
    WRITE_ATTRIBUTES = SpecificRight(code="WA", description="write attributes")


class InheritanceRights:
    """Options for how folders inherit rights"""

    OBJECT_INHERIT = ScriptParameter(code="(OI)", description="object inherit")
    CONTAINER_INHERIT = ScriptParameter(code="(CI)", description="container inherit")
    INHERIT_ONLY = ScriptParameter(code="(IO)", description="inherit only")
    DONT_PROPAGATE_INHERIT = ScriptParameter(
        code="(NP)", description="don't propagate inherit"
    )
    INHERIT_FROM_PARENT = ScriptParameter(
        code="(I)", description="permission inherited from parent container"
    )


class RightsCollection:
    """Understandable collection of simpler rights. For reasoning about rights without the hassle of
    having to know about 20 different specific permissions and options"""

    def __init__(
        self,
        specific_rights: Set[SpecificRight],
        description: str,
        inheritance_rights: Set[ScriptParameter],
    ):
        """

        Parameters
        ----------
        specific_rights: set(SpecificRight)
            set of specific rights that control which access is granted
        description: str
            human readable description of this right
        inheritance_rights: set(ScriptParameter)
            parameters that govern inheritance or rights within this folder
        """
        self.specific_rights = specific_rights
        self.inheritance_rights = inheritance_rights
        self.description = description


FULL_ACCESS = RightsCollection(
    specific_rights=set([SpecificRights.GENERIC_ALL]),
    description="Everything: read,write and delete",
    inheritance_rights=set(
        [InheritanceRights.OBJECT_INHERIT, InheritanceRights.CONTAINER_INHERIT]
    ),
)
READ_DELETE = RightsCollection(
    specific_rights=set(
        [
            SpecificRights.GENERIC_READ,
            SpecificRights.DELETE,
            SpecificRights.DELETE_CHILD,
        ]
    ),
    description="Read and delete files and folders, but not edit",
    inheritance_rights=set(
        [InheritanceRights.OBJECT_INHERIT, InheritanceRights.CONTAINER_INHERIT]
    ),
)
